/*
+-----------------------------------------------------------------+
| NAME:                                                           |
| util_addCensusRegnDiv                                           |
|                                                                 |
| TYPE:                                                           |
| SAS Macro code                                                  |
|                                                                 |
| DESCRIPTION:                                                    |
| This macro adds two columns (region & division) containing the  |
| the Census Bureau's Region and Division classification to the   |
| passed in data set.                                             |
|                                                                 |
| SAS VERSION: This software unit was developed for version 9.2   |
| (TS2M3).                                                        |
|                                                                 |
| HISTORY:                                                        |
| alatt001 10Mar2011   initial coding                             |
+-----------------------------------------------------------------+
*/


%macro util_addCensusRegnDiv
( p_dsName=			/* Table name to be modified */
, p_stateVarName=	/* The State Variable name - Expecting a numeric value */
);

	%local	l_rc
			l_msg
			l_sTime
			l_eTime
			l_rTime
			;

	/* -- Ensure All Required Parameters have been provided -- */
	%if (%superq(p_dsName) EQ  ) %then
	%do;
		%let l_rc  = 1;
		%let l_msg = A valid non missing Data set Name must be supplied!;
		%goto EXIT;
	%end;
	%else %if (%sysfunc(exist(%superq(p_dsName),data)) NE 1) %then
	%do;
		%let l_rc  = 2;
		%let l_msg = Specified Data set does not exist!;
		%goto EXIT;
	%end;

	%if (%superq(p_stateVarName) EQ  ) %then
	%do;
		%let l_rc  = 3;
		%let l_msg = A valid non missing variable name or expression must be supplied!;
		%goto EXIT;
	%end;
 	/* -- Ensure All Required Parameters have been provided -- */

 	/* -- Start the Processing -- */
	%let l_sTime = %sysfunc(datetime());

	DATA &p_dsName(DROP=ST REG DIV);

		SET	&p_dsName;

		LENGTH st $2 region $9 division $18 reg div 4 ;

		LABEL	region   = 'Census Region'
				division = 'Census Division'
				;

		/* Add 2-character state abbreviation, which is more readable */
		if (&p_stateVarName NE 0) then
			st=fipstate(&p_stateVarName);
		else
			st='';

		/* Assign divisions AND regions */
		if (st in ('CT' 'MA' 'RI' 'NH' 'VT' 'ME')) then
		do;
			div=1;
			division='New England';
			reg=1;
			region='Northeast';
		end;
		else if (st in ('PA' 'NJ' 'NY')) then
		do;
			div=2;
			division='Middle Atlantic';
			reg=1;
			region='Northeast';
		end;
		else if (st in ('OH' 'IN' 'IL' 'MI' 'WI')) then
		do;
			div=3;
			division='East North Central';
			reg=2;
			region='Midwest';
		end;
		else if (st in ('IA' 'KS' 'MN' 'MO' 'NE' 'ND' 'SD')) then
		do;
			div=4;
			division='West North Central';
			reg=2;
			region='Midwest';
		end;
		else if (st in ('DE' 'DC' 'FL' 'GA' 'MD' 'NC' 'SC' 'VA' 'WV')) then
		do;
			div=5;
			division='South Atlantic';
			reg=3;
			region='South';
		end;
		else if (st in ('AL' 'KY' 'MS' 'TN')) then
		do;
			div=6;
			division='East South Central';
			reg=3;
			region='South';
		end;
		else if (st in ('AR' 'LA' 'OK' 'TX')) then
		do;
			div=7;
			division='West South Central';
			reg=3;
			region='South';
		end;
		else if (st in ('AZ' 'CO' 'ID' 'NM' 'MT' 'UT' 'NV' 'WY')) then
		do;
			div=8;
			division='Mountain';
			reg=4;
			region='West';
		end;
		else if (st in ('AK' 'CA' 'HI' 'OR' 'WA')) then
		do;
			div=9;
			division='Pacific';
			reg=4;
			region='West';
		end;
	RUN;

	%goto FINISHED;

	%EXIT:
		%put;
		%put *** ERROR: util_addCensusRegnDiv ***;
		%put *** rc = &l_rc            ***;
		%put *** msg = &l_msg ***;
		%put *** ERROR: util_addCensusRegnDiv ***;
		%put;

	%FINISHED:
		%if (&l_sTime NE ) %then
		%do;
			%let l_eTime = %sysfunc(datetime());
			%let l_rTime = %sysevalf(&l_eTime - &l_sTime);
			%put;
			%put *** INFO: util_addCensusRegnDiv ***;
			%put *** Total Runtime = %sysfunc(putn(&l_rTime,time12.2)) ***;
			%put *** INFO: util_addCensusRegnDiv ***;
			%put;
		%end;

%mend util_addCensusRegnDiv;
